import React, { useState } from 'react'
import { useDrop } from 'react-dnd'
import { is, fromJS } from 'immutable'
import update from 'immutability-helper'
import { Col, Button, Popover, message } from 'antd'
import { EditOutlined } from '@ant-design/icons'

import Utils from '@/utils/utils.js'
import Card from './card'
import './index.scss'

const Container = ({list, setting, handleList, handleMenu, deleteMenu, handleSetting }) => {
  const [cards, setCards] = useState(list)
  const moveCard = (id, atIndex) => {
    const { card, index } = findCard(id)
    const _cards = update(cards, { $splice: [[index, 1], [atIndex, 0, card]] })
    handleList(_cards)
  }

  if (!is(fromJS(cards), fromJS(list))) {
    setCards(list)
  }
  
  const findCard = id => {
    const card = cards.filter(c => `${c.uuid}` === id)[0]
    return {
      card,
      index: cards.indexOf(card),
    }
  }

  const editCard = id => {
    const { card } = findCard(id)
    handleMenu(card)
  }
  

  const delCard = id => {
    const { card } = findCard(id)
    deleteMenu(card)
  }

  const copyCard = id => {
    const { card } = findCard(id)
    let copycard = fromJS(card).toJS()

    copycard.uuid = Utils.getuuid()
    copycard.origin = false
    copycard.copyType = 'search'
    copycard.focus = true

    let _val = fromJS(copycard).toJS()

    try {
      _val.uuid = Utils.getuuid()
      _val = window.btoa(window.encodeURIComponent(JSON.stringify(_val)))
    } catch (e) {
      console.warn('Stringify Failure')
      _val = ''
    }

    if (_val) {
      let oInput = document.createElement('input')
      oInput.value = _val
      document.body.appendChild(oInput)
      oInput.select()
      document.execCommand('Copy')
      document.body.removeChild(oInput)
    }

    if (window.GLOB.CopyOnly) {
      message.success('复制成功。')
    } else {
      const { index: overIndex } = findCard(id)

      const _cards = update(cards, { $splice: [[overIndex + 1, 0, copycard]] })

      handleList(_cards, copycard)
    }
  }

  const [, drop] = useDrop({
    accept: 'search',
    drop(item) {
      if (item.hasOwnProperty('originalIndex')) {
        return
      }

      let newcard = {}
      newcard.uuid = Utils.getuuid()
      newcard.focus = true

      newcard.label = 'label'
      newcard.initval = ''
      newcard.type = item.subType
      newcard.resourceType = '0'
      newcard.options = []
      newcard.orderType = 'asc'
      
      let _match = 'like'
      if (['select', 'link', 'checkcard'].includes(item.subType)) {
        _match = '='
      } else if (['date', 'datemonth'].includes(item.subType)) {
        _match = '>='
      } else if (['dateweek', 'daterange', 'range'].includes(item.subType)) {
        _match = 'between'
      } else if (item.subType === 'group') {
        _match = 'between'
        newcard.items = ['day', 'week', 'month', 'quarter', 'year', 'customized']
      }
      
      newcard.match = _match

      let targetId = ''

      if (item.dropTargetId) {
        targetId = item.dropTargetId
        delete item.dropTargetId
      } else if (cards.length > 0) {
        targetId = cards[cards.length - 1].uuid
      }

      const { index: overIndex, card } = findCard(`${targetId}`)

      if (card) {
        newcard.ratio = card.ratio || 6
        newcard.labelwidth = card.labelwidth || 33.3
      }

      const _cards = update(cards, { $splice: [[overIndex + 1, 0, newcard]] })

      handleList(_cards, newcard)
    }
  })

  let radio = setting.searchRatio || 6
  let labelwidth = setting.searchLwidth !== undefined ? setting.searchLwidth : 33.3
  let advanceType = setting.advanceType || 'modal'
  let style = {}
  if (setting.borderRadius) {
    style.borderRadius = setting.borderRadius
  }

  return (
    <div ref={drop} className="ant-row">
      {cards.map(card => {
        let _ratio = card.ratio || 6
        if (card.advanced === 'true' && advanceType !== 'pulldown') {
          _ratio = 6
        }
        return (
          <Col className="mk-search-item-wrap" key={card.uuid} span={_ratio}>
            <Card
              id={`${card.uuid}`}
              card={card}
              moveCard={moveCard}
              copyCard={copyCard}
              editCard={editCard}
              delCard={delCard}
              findCard={findCard}
            />
          </Col>
        )
      })}
      {cards.length > 0 ? <Col className={'mk-search-item-wrap action ' + (setting.show === 'false' ? 'hide-button' : '')} key="action" span={radio}>
        <div className="ant-row ant-form-item" style={{whiteSpace: 'nowrap', lineHeight: '40px', height: '55px', marginBottom: 0}}>
          <div className="ant-col ant-form-item-label" style={{width: labelwidth + '%'}}></div>
          <Popover overlayClassName="mk-popover-control-wrap" mouseLeaveDelay={0.2} mouseEnterDelay={0.2} content={
            <div className="mk-popover-control">
              <EditOutlined className="edit" onClick={() => handleSetting()} />
            </div>
          } trigger="hover">
            <div className="ant-col ant-form-item-control-wrapper">
              <Button style={style} type="primary">搜索</Button>
              <Button style={{ marginLeft: 8, ...style }}>重置</Button>
              <div style={{position: 'absolute', top: 0, bottom: 0, left: 0, right: 0}}></div>
            </div>
          </Popover>
        </div>
      </Col> : null}
      {cards.length === 0 ?
        <div className="common-drawarea-placeholder">
          请添加搜索条件
        </div> : null
      }
    </div>
  )
}
export default Container
